#ifndef DESGIN_H_INCLUDED_
#define DESGIN_H_INCLUDED_

#include "rose.h"

#include "module.h"
#include "primitive_channel.h"
#include "hierarchical_channel.h"

#include "event.h"

#include "thread.h"
#include "cthread.h"
#include "method.h"
#include "function.h"

#include "instance_tree.h"
#include "path_instance_mapper.h"

#include "../tools/systemc_definition_finder.h"

namespace risc {

/**
 * This class represents the internal representation. The IR offers different
 * interfaces to start search in IR.
 */
class Design {

public:

  /**
   * \brief default constructor
   * \param argc  expects argc from main function
   * \param argv  expects argv from main function
   */
  explicit Design(int argc, char* argv[]);
  ~Design();

  void initialize_defintions();
  void initialize_top_modules();
  void initialize_global_functions();

  void initialize_global_variables();

  /**
   * \brief Find a port by it's ast variable definition
   */
  Variable* find_global_variable(SgVariableDefinition* var_def);

  // TODO No implementation available
  void initialize_sc_main_variables();

  // TODO No implementation available
  void initialize_sc_main_channels();


  //////////////////////// Functions to print vectors ////////////////////////
  void print_all_class_definitions();
  void print_all_global_functions();
  void print_design();

  /////////////////////////////// Instance Tree //////////////////////////////
  InstanceTree get_root();

  /////////////////// Vectors of different design elements ///////////////////
  ModuleVector module_definitions_;
  ModuleInstanceVector top_modules_;

  PrimitiveChannelVector primitive_channel_;
  HierarchicalChannelVector hierarchical_channel_;

  EventVector events_;
  VariableVector variables_; // global variables

  FunctionVector functions_;

  ///////////////// global and sc_main related design stuff //////////////////
  VariableVector global_variables_;
  FunctionVector global_functions_;

  /////////////////////// Functions to search in design //////////////////////
  Module *find_module_definition(SgClassDefinition* cd);

  /////////////////////////// Rose specific elements /////////////////////////
  SgProject *project_;
  SgFunctionDeclaration* main_;

  ////////////////////////// SystemC specific elements ///////////////////////
  /**
   * \brief Pointer to the sc_module definition in the rose ast tree
   */
  risc::tools::SystemCDefinitionFinder *systemc_definitions_;

  /**
   * \brief Pointer to the sc_main function declaration
   */
  SgFunctionDeclaration *sc_main_;

  //////////////////////////////////// misc //////////////////////////////////
  int old_argc_;
  char **old_argv_;

  std::vector<std::string> input_files_;

  PathInstanceMapper path_instance_mapper_;

  /**
   * \function reset_instance_counter
   * \brief    This function resets the instance counter of the module and
   *           hierarchical channel instances to zero.
   */
  void reset_instance_counter();

private:
  /**
   * \brief Disabled copy constructor
   */
  Design(const Design &d);
};

class GlobalDesign {
public:
  static Design *design_static;
};
Design* get_design();
SgProject* get_project();

}; // end namespace risc

#endif /* DESGIN_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
