#include "data_conflict_table.h"

#include <fstream>
#include "../tools/variable_helper.h"


risc::sg::DataConflictTable::DataConflictTable(
    SegmentGraph &graph,
    PathInstanceMapper &path_instance_mapper,
    bool debugging):
  ConflictTable(graph, path_instance_mapper, false)
{
  // here we perform a read/write analysis on each segment in the graph
  std::pair<VertexIterator, VertexIterator> vp;
  for(vp = boost::vertices(segment_graph_.graph_);
      vp.first != vp.second; ++vp.first) {

    segment_graph_.graph_[*vp.first].analyze_segment();
  }

  determine_conflict_table();
}

void
risc::sg::DataConflictTable::determine_conflict_table()
{
  for(int i = 0; i < size_of_conflict_table_; i ++) {

    for(int j = 0; j < size_of_conflict_table_; j ++) {
      conflict_table_[i][j] = true;
    }
  }

//  for(int row=0; row < size_; row++) {
//
//    for(int column=0; column<=row; column++) {
//
//      if(row == column) {
//        // If only read access in this segment, there will be no conflict
//        if(segment_graph_.graph_[*table_index_segment_lookup_.at(column)].
//            write_variables_.empty()) {
//
//          table_[row][column] = false;
//        } else {
//
//          bool has_conflict = false;
//          for(std::set<SgVariableSymbol*>::iterator
//              write_iter  = segment_graph_.graph_[*table_index_segment_lookup_.
//                                at(row)].write_variables_.begin();
//              write_iter != segment_graph_.graph_[*table_index_segment_lookup_.
//                                at(row)].write_variables_.end();
//              write_iter++) {
//
//            if((tools::is_member_variable(*write_iter)) ||
//
//                (tools::is_local_variable(*write_iter) &&
//                tools::is_static_variable(*write_iter)) ||
//
//                tools::is_global_variable(*write_iter) ||
//                tools::is_namespace_variable(*write_iter)) {
//              has_conflict = true;
//
//              if(debugging_) {
//                conflict_info_[column][row].conflict_variables_.
//                  insert(*write_iter);
//                conflict_info_[row][column].conflict_variables_.
//                  insert(*write_iter);
//              }
//            }
//          }
//
//          table_[row][column] = has_conflict;
//        }
//
//        if(segment_graph_.graph_[*table_index_segment_lookup_.at(row)].
//            in_conflict_with_all_other_segments){
//
//          conflict_info_[row][column].conflict_variables_ =
//            segment_graph_.graph_[*table_index_segment_lookup_.at(row)].
//              conflicts_;
//        }
//
//        table_[row][column] |=
//          segment_graph_.graph_[*table_index_segment_lookup_.at(row)].
//              in_conflict_with_all_other_segments;
//
//      } else {
//
//        table_[row][column] = data_conflict_between(
//                                  table_index_segment_lookup_.at(row),
//                                  table_index_segment_lookup_.at(column),
//                                  row, column);
//        table_[column][row] = table_[row][column];
//
//
//        conflict_info_[column][row].conflict_ = table_[row][column];
//        conflict_info_[row][column].conflict_ = table_[row][column];
//      }
//    }
//  }
}

void
risc::sg::DataConflictTable::color_members_of_class(
    SgVariableSymbol *var_symbol,
    DataConflictAstAttributes::Status color)
{
  // we get the type of the variable
  SgInitializedName *var_decl_stmt = var_symbol->get_declaration();
  SgType *type = var_decl_stmt->get_type();
  SgDeclarationStatement *type_decl_stmt = type->getAssociatedDeclaration();

  if(!type_decl_stmt) {
    return;
  }

  SgClassDeclaration *class_decl = isSgClassDeclaration(type_decl_stmt);

  if(!class_decl) {
    return;
  }

  class_decl = isSgClassDeclaration(class_decl->get_definingDeclaration());

  SgClassDefinition *class_def = class_decl->get_definition();

  for(Rose_STL_Container<SgDeclarationStatement*>::iterator
      member_iter  = class_def->get_members().begin();
      member_iter != class_def->get_members().end();
      member_iter++) {

    SgVariableDeclaration *var_decl = isSgVariableDeclaration(*member_iter);
    if(var_decl) {

      if(!var_decl->get_decl_item(0)->get_storageModifier().isStatic()) {


        SgVariableSymbol *var_symbol =
          isSgVariableSymbol(var_decl->get_definition()->get_vardefn()->
                              search_for_symbol_from_symbol_table());

        DataConflictAstAttributes *data_conflict_attribute =
          dynamic_cast<DataConflictAstAttributes*>(
              var_symbol->getAttribute("data_conflict"));

        if(data_conflict_attribute == NULL) {
          data_conflict_attribute = new DataConflictAstAttributes();
          data_conflict_attribute->status_ = color;
          var_symbol->setAttribute("data_conflict", data_conflict_attribute);

        } else {
          data_conflict_attribute->status_ = color;

        }
      }
    }
  }
}

void
risc::sg::DataConflictTable::uncolor_members_of_class(
    SgVariableSymbol *var_symbol)
{
  // we get the type of the variable
  SgInitializedName *var_decl_stmt = var_symbol->get_declaration();
  SgType *type = var_decl_stmt->get_type();
  SgDeclarationStatement *type_decl_stmt = type->getAssociatedDeclaration();

  if(!type_decl_stmt) {
    return;
  }

  SgClassDeclaration *class_decl = isSgClassDeclaration(type_decl_stmt);

  if(!class_decl) {
    return;
  }

  class_decl = isSgClassDeclaration(class_decl->get_definingDeclaration());

  SgClassDefinition *class_def = class_decl->get_definition();

  for(Rose_STL_Container<SgDeclarationStatement*>::iterator
      member_iter  = class_def->get_members().begin();
      member_iter != class_def->get_members().end();
      member_iter++) {

    SgVariableDeclaration *var_decl = isSgVariableDeclaration(*member_iter);
    if(var_decl) {

      if(!var_decl->get_decl_item(0)->get_storageModifier().isStatic()) {


        SgVariableSymbol *var_symbol =
          isSgVariableSymbol(var_decl->get_definition()->get_vardefn()->
                              search_for_symbol_from_symbol_table());

        DataConflictAstAttributes *data_conflict_attribute =
          dynamic_cast<DataConflictAstAttributes*>(
              var_symbol->getAttribute("data_conflict"));

        if(data_conflict_attribute != NULL) {

          data_conflict_attribute->status_ = DataConflictAstAttributes::UNUSED;
        }
      }
    }
  }
}

bool
risc::sg::DataConflictTable::check_members_of_class(
    SgVariableSymbol *var_symbol,
    DataConflictAstAttributes::Status color)
{
  SgInitializedName *var_decl_stmt = var_symbol->get_declaration();
  SgType *type = var_decl_stmt->get_type();
  SgDeclarationStatement *type_decl_stmt = type->getAssociatedDeclaration();

  if(!type_decl_stmt) {
    return false;
  }

  // we get the type of the variable
  SgClassDeclaration *class_decl = isSgClassDeclaration(type_decl_stmt);

  if(!class_decl) {
    return false;
  }

  class_decl = isSgClassDeclaration(class_decl->get_definingDeclaration());

  SgClassDefinition *class_def = class_decl->get_definition();

  for(Rose_STL_Container<SgDeclarationStatement*>::iterator
      member_iter  = class_def->get_members().begin();
      member_iter != class_def->get_members().end();
      member_iter++) {

    SgVariableDeclaration *var_decl = isSgVariableDeclaration(*member_iter);
    if(var_decl) {
      if(!var_decl->get_decl_item(0)->get_storageModifier().isStatic()) {

        DataConflictAstAttributes *data_conflict_attribute =
          dynamic_cast<DataConflictAstAttributes*>(
              var_symbol->getAttribute("data_conflict"));

        if(data_conflict_attribute != NULL) {

          if(data_conflict_attribute->status_ == color) {
            return true;
          }
        }
      }
    }
  }
  return false;
}


bool
risc::sg::DataConflictTable::data_conflict_between(VertexIterator first,
    VertexIterator second, int row, int column)
{
  if(segment_graph_.graph_[*second].in_conflict_with_all_other_segments
      || segment_graph_.graph_[*first].in_conflict_with_all_other_segments) {

    if(segment_graph_.graph_[*first].in_conflict_with_all_other_segments) {

      conflict_info_[row][column].conflict_variables_.insert(
          segment_graph_.graph_[*first].conflicts_.begin(),
          segment_graph_.graph_[*first].conflicts_.end() );

      conflict_info_[column][row].conflict_variables_.insert(
          segment_graph_.graph_[*first].conflicts_.begin(),
          segment_graph_.graph_[*first].conflicts_.end() );
    }

    if(segment_graph_.graph_[*second].in_conflict_with_all_other_segments) {

      conflict_info_[row][column].conflict_variables_.insert(
          segment_graph_.graph_[*second].conflicts_.begin(),
          segment_graph_.graph_[*second].conflicts_.end() );

      conflict_info_[column][row].conflict_variables_.insert(
          segment_graph_.graph_[*second].conflicts_.begin(),
          segment_graph_.graph_[*second].conflicts_.end() );
    }

    return true;
  }

  std::set<SgNode*> conflicts;

  bool conflict = false;

  for(std::set<SgVariableSymbol*>::iterator
      read_iter  = segment_graph_.graph_[*first].read_variables_.begin();
      read_iter != segment_graph_.graph_[*first].read_variables_.end();
      read_iter++) {

    if((tools::is_local_variable(*read_iter) &&
        !tools::is_static_variable(*read_iter)) ||
        tools::is_function_parameter_variable(*read_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*read_iter)->getAttribute("data_conflict"));

    if(data_conflict_attribute == NULL) {
      data_conflict_attribute = new DataConflictAstAttributes();
      data_conflict_attribute->status_ = DataConflictAstAttributes::READ;
      (*read_iter)->setAttribute("data_conflict", data_conflict_attribute);

    } else {
      data_conflict_attribute->status_ = DataConflictAstAttributes::READ;
    }

    // In the case that we are reading a non pod data type we have to mark
    // all class/struct members as read
    color_members_of_class(*read_iter, DataConflictAstAttributes::READ);
  }

  for(std::set<SgVariableSymbol*>::iterator
      write_iter  = segment_graph_.graph_[*first].write_variables_.begin();
      write_iter != segment_graph_.graph_[*first].write_variables_.end();
      write_iter++) {

    if((tools::is_local_variable(*write_iter) &&
        !tools::is_static_variable(*write_iter)) ||
        tools::is_function_parameter_variable(*write_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*write_iter)->getAttribute("data_conflict"));

    if(data_conflict_attribute == NULL) {
      data_conflict_attribute = new DataConflictAstAttributes();
      data_conflict_attribute->status_ = DataConflictAstAttributes::WRITE;
      (*write_iter)->setAttribute("data_conflict", data_conflict_attribute);

    } else {
      data_conflict_attribute->status_ = DataConflictAstAttributes::WRITE;
    }

    // In the case that we are reading a non pod data type we have to mark
    // all class/struct members as write
    color_members_of_class(*write_iter, DataConflictAstAttributes::WRITE);
  }

  for(std::set<SgVariableSymbol*>::iterator
      read_iter  = segment_graph_.graph_[*second].read_variables_.begin();
      read_iter != segment_graph_.graph_[*second].read_variables_.end();
      read_iter++) {


    if((tools::is_local_variable(*read_iter) &&
        !tools::is_static_variable(*read_iter)) ||
        tools::is_function_parameter_variable(*read_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*read_iter)->getAttribute("data_conflict"));

    if(data_conflict_attribute != NULL) {

      if(data_conflict_attribute->status_ == DataConflictAstAttributes::WRITE) {

        conflict = true;
        if(debugging_) {
          conflicts.insert(*read_iter);
        } else {
          break;
        }
      }
    }

    conflict |= check_members_of_class(*read_iter,
                    DataConflictAstAttributes::WRITE);
  }

  for(std::set<SgVariableSymbol*>::iterator
      write_iter  = segment_graph_.graph_[*second].write_variables_.begin();
      write_iter != segment_graph_.graph_[*second].write_variables_.end();
      write_iter++) {

    if((tools::is_local_variable(*write_iter) &&
        !tools::is_static_variable(*write_iter)) ||
        tools::is_function_parameter_variable(*write_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*write_iter)->getAttribute("data_conflict"));

    if(data_conflict_attribute != NULL) {

      if(data_conflict_attribute->status_ !=
          DataConflictAstAttributes::UNUSED) {

        conflict = true;

        if(debugging_) {
          conflicts.insert(*write_iter);
        } else {
          break;
        }
      }
    }

    conflict |= check_members_of_class(*write_iter,
                    DataConflictAstAttributes::WRITE);
    conflict |= check_members_of_class(*write_iter,
                    DataConflictAstAttributes::READ);
  }


  if(!conflict) {

    for(std::set<SgVariableSymbol*>::iterator
        write_iter  = segment_graph_.graph_[*second].write_variables_.begin();
        write_iter != segment_graph_.graph_[*second].write_variables_.end();
        write_iter++) {

      if((tools::is_local_variable(*write_iter) &&
          !tools::is_static_variable(*write_iter)) ||
          tools::is_function_parameter_variable(*write_iter)) {
        continue;
      }

      DataConflictAstAttributes *data_conflict_attribute =
        dynamic_cast<DataConflictAstAttributes*>(
            (*write_iter)->getAttribute("data_conflict"));

      if(data_conflict_attribute != NULL) {

        if(data_conflict_attribute->status_
            != DataConflictAstAttributes::UNUSED) {
          conflict = true;

          if(debugging_) {
            conflicts.insert(*write_iter);
          } else {
            break;
          }
        }
      }
    }
  }

  for(std::set<SgVariableSymbol*>::iterator
      read_iter  = segment_graph_.graph_[*first].write_variables_.begin();
      read_iter != segment_graph_.graph_[*first].write_variables_.end();
      read_iter++) {

    if((tools::is_local_variable(*read_iter) &&
        !tools::is_static_variable(*read_iter)) ||
        tools::is_function_parameter_variable(*read_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*read_iter)->getAttribute("data_conflict"));
    data_conflict_attribute->status_ = DataConflictAstAttributes::UNUSED;

    uncolor_members_of_class(*read_iter);
  }

  for(std::set<SgVariableSymbol*>::iterator
      read_iter  = segment_graph_.graph_[*first].write_variables_.begin();
      read_iter != segment_graph_.graph_[*first].write_variables_.end();
      read_iter++) {

    if((tools::is_local_variable(*read_iter) &&
        !tools::is_static_variable(*read_iter)) ||
        tools::is_function_parameter_variable(*read_iter)) {
      continue;
    }

    DataConflictAstAttributes *data_conflict_attribute =
      dynamic_cast<DataConflictAstAttributes*>(
          (*read_iter)->getAttribute("data_conflict"));
    data_conflict_attribute->status_ = DataConflictAstAttributes::UNUSED;

    uncolor_members_of_class(*read_iter);
  }

  if(debugging_) {
    if(conflict) {

      for(std::set<SgNode*>::iterator
          iter  = conflicts.begin();
          iter != conflicts.end();
          iter++) {
        SgVariableSymbol *var_sym = isSgVariableSymbol(*iter);
      }
    }
  }

  if(debugging_) {
    conflict_info_[row][column].conflict_variables_ = conflicts;
    conflict_info_[column][row].conflict_variables_ = conflicts;
  }

  return conflict;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
