#ifndef EVENT_CONFLICT_TABLE_H_INCLUDED_
#define EVENT_CONFLICT_TABLE_H_INCLUDED_

#include "segment_graph.h"
#include "segment.h"
#include "conflict_table.h"
#include "../internal_representation/path_instance_mapper.h"

namespace risc {

namespace sg {

/**
 * \class EventConflictTable
 * \brief The conflicts are stored in the base class variable conflict_table_.
 *        The columns represent wait statements of the segments. The rows are
 *        representing the notifications of events. So, a true in
 *        conflict_table_[i][j] means that the associated segment/instance i
 *        notifies the event for which the associated segment/instance j is
 *        waiting.
 *
 * TODO   Handling for pointers
 */
class EventConflictTable: public ConflictTable {

public:

  /**
   * \brief    Default constructor for the DataConflictTable
   */
  EventConflictTable(
      SegmentGraph &graph,
      PathInstanceMapper &path_instance_mapper,
      bool debugging);

  /**
   * \function determine_conflict_table
   * \brief    This function determines conflicts among the segments
   */
  virtual void determine_conflict_table();
};

}; // end of namesapce sg

}; // end of namespace risc


#endif /* EVENT_CONFLICT_TABLE_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
