#ifndef READ_WRITE_ANALYSIS_H_INCLUDED_
#define READ_WRITE_ANALYSIS_H_INCLUDED_

#include <rose.h>

#include <vector>
#include <cassert>

#include "../internal_representation/variable.h"

namespace risc
{

namespace sa
{

/**
 * \function collect_write_variables
 * \brief    This function collects all written variables in the given
 *           expression and adds them to the given vector
 */
void
collect_write_variables(SgExpression *expr, std::vector<SgNode*> &write_var);

/**
 * \function collect_read_variables
 * \brief    This function collects all read variables in the given
 *           expression and adds them to the given vector
 */
void
collect_read_variables( SgExpression *expr, std::vector<SgNode*> &read_var);

/**
 * \function read_write_analysis
 * \brief    This function collects all read only, write only, and read/write
 *           variables in the given expression vector and adds them to the
 *           corresponging set/vector
 */
void
read_write_analysis(
    std::vector<SgExpression*> expressions,
    std::set<SgVariableSymbol*> &read_only_var,
    std::set<SgVariableSymbol*> &write_only_var,
    std::vector<SgNode*> &read_write);

bool
is_member_variable_symbol(SgNode *given_variable,
    risc::VariableVector &member_variables);

std::vector<SgNode*>::iterator
delete_if_is_not_member_variable(
    std::vector<SgNode*> &given_variables,
    VariableVector &member_variables);



std::set<SgVariableSymbol*>::iterator
delete_if_is_not_member_symbol(
    std::set<SgVariableSymbol*> &given_variables,
    VariableVector &member_variables);




} // end namesapce sa

} // end namespace risc

#endif /* READ_WRITE_ANALYSIS_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
