#ifndef CLASS_IS_DERIVED_FROM_H_INCLUDED_
#define CLASS_IS_DERIVED_FROM_H_INCLUDED_

#include "rose.h"

namespace risc {

namespace tools {

/**
  * \brief               This function checks if the derived class is derived
  *                      from the base class
  * \param derived_class the derived class definition
  * \param base_class    the base class definition
  * \return              true, if the class defintion derived_class is derived
  *                      from calls definition base_class. Otherwise false
  */
bool
is_derived_from (SgClassDefinition* derived_class,
    SgClassDefinition* base_class);


/**
  * \brief               This function checks if the class is derived from
  *                      sc_module
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_module
  */
bool
is_derived_from_sc_module(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_port
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_port
  */
bool
is_derived_from_sc_port(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_port_base
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_port_base
  */
bool
is_derived_from_sc_port_base(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_port_base
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_port_base
  */
bool
is_derived_from_sc_port_base(SgVarRefExp* var_ref);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_prim_channel
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_prim_channel
  */
bool
is_derived_from_sc_prim_channel(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_channel
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_channel
  */
bool
is_derived_from_sc_channel(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_event or if it is sc_event itself
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_event
  */
bool
is_sc_event(SgClassDefinition* derived_class);

/**
  * \brief               This function checks if the class is derived from
  *                      sc_event or if it is sc_event itself
  * \param derived_class the derived class definition
  * \return              True, if the class is derived from sc_event
  */
bool
is_sc_event(SgVarRefExp* var_ref);

/**
  * \brief               This function checks if the template class bases on
  *                      the class template sc_in_port
  *
  * \param    class      the class definition
  * \return              True, if the template class bases on the class
  *                      template sc_in_port
  */
bool
is_sc_in_port(SgTemplateInstantiationDefn* template_class);

/**
  * \brief               This function checks if the template class bases on
  *                      the class template sc_out_port
  *
  * \param    class      the class definition
  * \return              True, if the template class bases on the class
  *                      template sc_out_port
  */
bool
is_sc_out_port(SgTemplateInstantiationDefn* template_class);

/**
  * \brief               This function checks if the template class bases on
  *                      the class template sc_inout_port
  *
  * \param    class      the class definition
  * \return              True, if the template class bases on the class
  *                      template sc_inout_port
  */
bool
is_sc_inout_port(SgTemplateInstantiationDefn* template_class);

/**
  * \brief               This function checks if the given defintion is qual
  *                      to the definition of a sc_in_clk
  *
  * \param    class      the class definition
  * \return              True, if equal
  */
bool
is_sc_in_clk(SgClassDefinition* definition);

/**
  * \brief               This function checks if the given defintion is qual
  *                      to the definition of a sc_out_clk
  *
  * \param    class      the class definition
  * \return              True, if equal
  */
bool
is_sc_out_clk(SgClassDefinition* definition);

/**
  * \brief               This function checks if the given defintion is qual
  *                      to the definition of a sc_inout_clk
  *
  * \param    class      the class definition
  * \return              True, if equal
  */
bool
is_sc_inout_clk(SgClassDefinition* definition);

/**
  * \brief               This function checks if the given defintion is qual
  *                      to the definition of a sc_in<bool>
  *
  * \param    class      the class definition
  * \return              True, if equal
  */
bool
is_sc_in_bool(SgClassDefinition* definition);

} // end namespace tools

} // end namespace risc

#endif /* CLASS_IS_DERIVED_FROM_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
