#include <iostream>

#include<utility>
#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl
            << "start processing file: " __FILE__
            << " ..."
            << std::endl;

  risc::Design d(argc,argv);


  risc::sg::SegmentGraph::FunctionCallBoundarySet function_call_boundary(
      d.systemc_definitions_->sc_wait_funcs_);
  risc::sg::SegmentGraph sg(function_call_boundary);

  for(risc::ModuleVectorIter
      iter_module  = d.module_definitions_.begin();
      iter_module != d.module_definitions_.end();
      iter_module++) {

    std::cout << "Module: " <<  (*iter_module)->get_name() << std::endl;

    for(risc::ThreadVectorIter
        iter_thread = (*iter_module)->threads_.begin();
        iter_thread != (*iter_module)->threads_.end();
        iter_thread++) {

      std::cout << "  " << (*iter_thread)->get_name() << std::endl;


      sg.add_function(*iter_thread);

      //AstDOTGeneration dotgen;
      //dotgen.generate((*iter_thread)->get_ast_node(), (*iter_thread)->get_name());
    }

    std::cout << std::endl;
  }

  sg.clean_graph();
  sg.print_graph("conflict_table.dot");

  risc::sg::ConflictTable ct(sg, d.path_instance_mapper_, true);

  ct.determine_conflict_table();

  std::cout << "Size of the conflict table: "
            << ct.size_of_conflict_table_
            << std::endl;
  std::cout << "Number of Segments:         "
            << ct.number_of_segments_
            << std::endl;
  std::cout << "Max number of instances:    "
            << ct.path_instance_mapper_.max_instances_
            << std::endl;
  std::cout << std::endl;

  ct.print_instance_id_to_index_lookup_table();
  ct.print_lookup_table("conflict_lookup_table.html");
  ct.print_conflict_table("conflict_table.html");

  std::cout << std::endl;


  std::cout << "Segment 13, instance 0 -> "
            << ct.segment_and_instance_id_to_index(13, 0)
            << std::endl;

  std::pair<int, int> segment_instance_id
    = ct.index_to_segment_and_instance_id(21);
  std::cout << "Index 21 -> "
            << "Segment " << segment_instance_id.first
            << ", Instance " << segment_instance_id.second
            << std::endl;

  segment_instance_id = ct.index_to_segment_and_instance_id(7);
  std::cout << "Index 7 -> "
            << "Segment " << segment_instance_id.first
            << ", Instance " << segment_instance_id.second
            << std::endl;

  segment_instance_id = ct.index_to_segment_and_instance_id(17);
  std::cout << "Index 17 -> "
            << "Segment " << segment_instance_id.first
            << ", Instance " << segment_instance_id.second
            << std::endl;

  std::cout << "end processing file: " __FILE__
            << " ..."
            << std::endl
            << std::endl;

  return 0;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
