#include <iostream>

#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl
            << "start processing file: " __FILE__
            << " ..."
            << std::endl;

  risc::Design d(argc,argv);

  // iterating all modules in the design
  for(risc::ModuleVectorIter
      iter_module  = d.module_definitions_.begin();
      iter_module != d.module_definitions_.end();
      iter_module++) {

    std::cout << "Module: " <<  (*iter_module)->get_name() << std::endl;

    for(risc::CThreadVectorIter
        iter_cthreads  = ((*iter_module)->cthreads_).begin();
        iter_cthreads != ((*iter_module)->cthreads_).end();
        iter_cthreads++) {

      std::cout << "  " << (*iter_cthreads)->get_name();
      SgNode *node = risc::sa::find_first_wait_stmt((*iter_cthreads)->get_ast_node());

      if(node) {
        std::cout << ", Line: " << node->get_file_info()->get_line() << std::endl;
      }

      SgFunctionCallExp *func_call = isSgFunctionCallExp(node);

      std::set<SgFunctionCallExp*> func_calls =
          risc::sa::traverse_for_non_nested_func_calls((*iter_cthreads)->get_ast_node());

      for(std::set<SgFunctionCallExp*>::iterator
          iter_func_call  = func_calls.begin();
          iter_func_call != func_calls.end();
          iter_func_call++) {

        std::set<SgFunctionSymbol*> func_symbols = risc::sa::ga::dfs(
            (*iter_func_call)->getAssociatedFunctionSymbol(),
            *(d.call_graph_builder_->getGraph()));

        std::cout << "    ";
        for(std::set<SgFunctionSymbol*>::iterator
            iter_nested_call  = func_symbols.begin();
            iter_nested_call != func_symbols.end();
            iter_nested_call++) {
          std::cout << SageInterface::get_name((*iter_nested_call)->get_declaration()) << " ";
        }
        std::cout << std::endl;
      }
      std::cout << std::endl;
    }
  }


  std::cout << "end processing file: " __FILE__
            << " ..."
            << std::endl
            << std::endl;

  return 0;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
